import express from 'express';
import logger from 'morgan';
import Bourne from 'bourne';
import path from 'path';
import graphqlHTTP from 'express-graphql';
import {buildSchema} from 'graphql';
import {importSchema} from 'graphql-import';
import {connectionFromArray} from 'graphql-relay';
import {v4} from 'uuid';
import {promisify} from 'util';

const app = express();
const recipes = new Bourne('db/recipes.json');

app.use(logger('dev'));
app.use(express.static('public'));

// Construct a schema, using GraphQL schema language
const schemaDef = importSchema(path.resolve('server', 'schema.graphql'));
const schema = buildSchema(schemaDef);

// promisify db methods
const findRecipes = promisify(recipes.find.bind(recipes));
const findOneRecipe = promisify(recipes.findOne.bind(recipes));
const updateRecipe = promisify(recipes.update.bind(recipes));
const insertRecipe = promisify(recipes.insert.bind(recipes));
const deleteRecipe = promisify(recipes.delete.bind(recipes));

// The root provides a resolver function for each API endpoint
const root = {
  // Queries
  recipes: async (args) => {
    const conn = await findRecipes()
      .then(recipes => connectionFromArray(recipes, args));
    return conn;
  },
  // Mutations
  changeRecipeRate: async ({input}) => {
    const id = parseInt(input.id, 10);
    let recipe = await findOneRecipe({id});
    recipe = await updateRecipe({id}, {
      ...recipe,
      rating: input.rating
    }).then(results => results[0]);
    return {recipe};
  },
  addRecipe: async ({input}) => {
    const {name, rating} = input;
    const ingredients = [...input.ingredients];
    const instructions = [...input.instructions];
    const recipe = await insertRecipe({
      name,
      ingredients,
      instructions,
      rating
    });
    return {recipe};
  },
  removeRecipe: async ({input}) => {
    const id = parseInt(input.id, 10);
    await deleteRecipe({id});
    return {
      deletedRecipeId: id
    }
  }
};

app.use('/graphql', graphqlHTTP({
  schema,
  rootValue: root,
  graphiql: true
}));

app.listen(3000, () => console.log(`Application running at 'http://localhost:3000'`));
